"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = attacher;

var _unistUtilVisit = _interopRequireDefault(require("unist-util-visit"));

var _extend = _interopRequireDefault(require("extend"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(Object(source), true).forEach(function (key) { _defineProperty(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

const behaviors = {
  prepend: 'unshift',
  append: 'push'
};
const contentDefaults = {
  type: 'element',
  tagName: 'span',
  properties: {
    className: ['icon', 'icon-link']
  },
  children: []
};
const defaults = {
  behavior: 'prepend',
  content: contentDefaults
};
const splice = [].splice;
let deprecationWarningIssued = false;

function attacher(options = {}) {
  let {
    linkProperties,
    behavior,
    content,
    group
  } = _objectSpread(_objectSpread({}, defaults), options);

  let method; // NOTE: Remove in next major version

  if (options.behaviour !== undefined) {
    if (!deprecationWarningIssued) {
      deprecationWarningIssued = true;
      console.warn('[remark-autolink-headings] Deprecation Warning: `behaviour` is a nonstandard option. Use `behavior` instead.');
    }

    behavior = options.behaviour;
  }

  if (behavior === 'wrap') {
    method = wrap;
  } else if (behavior === 'before' || behavior === 'after') {
    method = around;
  } else {
    method = inject;

    if (!linkProperties) {
      linkProperties = {
        ariaHidden: 'true',
        tabIndex: -1
      };
    }
  }

  return tree => (0, _unistUtilVisit.default)(tree, 'heading', visitor);

  function visitor(node, index, parent) {
    const {
      data
    } = node;
    const id = data && data.hProperties && data.hProperties.id;

    if (id) {
      return method(node, '#' + id, index, parent);
    }
  }

  function inject(node, url) {
    const link = create(url);
    link.data = {
      hProperties: toProps(linkProperties),
      hChildren: toChildren(content, node)
    };
    node.children[behaviors[behavior]](link);
  }

  function around(node, url, index, parent) {
    const link = create(url);
    const grouping = group ? toGrouping(group, node) : undefined;
    link.data = {
      hProperties: toProps(linkProperties),
      hChildren: toChildren(content, node)
    };
    let nodes = behavior === 'before' ? [link, node] : [node, link];

    if (grouping) {
      grouping.children = nodes;
      nodes = grouping;
    }

    splice.apply(parent.children, [index, 1].concat(nodes));
    return [_unistUtilVisit.default.SKIP, index + nodes.length];
  }

  function wrap(node, url) {
    const link = create(url, node.children);
    link.data = {
      hProperties: toProps(linkProperties)
    };
    node.children = [link];
  }

  function toProps(value) {
    return deepAssign({}, value);
  }

  function toNode(value, node) {
    return typeof value === 'function' ? value(node) : value;
  }

  function toChildren(value, node) {
    let children = toNode(value, node);
    children = Array.isArray(children) ? children : [children];
    return typeof value === 'function' ? children : deepAssign([], children);
  }

  function toGrouping(value, node) {
    const grouping = toNode(value, node);
    const hName = grouping.tagName;
    const hProperties = grouping.properties;
    return {
      type: 'heading-group',
      data: {
        hName,
        hProperties: typeof value === 'function' ? deepAssign({}, hProperties) : hProperties
      },
      children: []
    };
  }

  function create(url, children) {
    return {
      type: 'link',
      url,
      title: null,
      children: children || []
    };
  }

  function deepAssign(base, value) {
    return (0, _extend.default)(true, base, value);
  }
}

module.exports = exports.default;