"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const Blockquote_1 = require("../Blockquote");
const Code_1 = require("../Code");
const Heading_1 = require("../Heading");
const HTML_1 = require("../HTML");
const List_1 = require("../List");
const Paragraph_1 = require("../Paragraph");
const Table_1 = require("../Table");
const ThematicBreak_1 = require("../ThematicBreak");
/**
 * Type guard for BlockContent.
 *
 * @param node - The node to test.
 * @returns True if node is BlockContent.
 */
function isBlockContent(node) {
    return (Paragraph_1.isParagraph(node) ||
        Heading_1.isHeading(node) ||
        ThematicBreak_1.isThematicBreak(node) ||
        Blockquote_1.isBlockquote(node) ||
        List_1.isList(node) ||
        Table_1.isTable(node) ||
        HTML_1.isHTML(node) ||
        Code_1.isCode(node));
}
exports.isBlockContent = isBlockContent;
/**
 * Type assertion for BlockContent.
 *
 * @param node - The node to test.
 */
function assertBlockContent(node) {
    if (!isBlockContent(node)) {
        throw new Error('Node is not a BlockContent');
    }
}
exports.assertBlockContent = assertBlockContent;
//# sourceMappingURL=BlockContent.js.map