"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
/*
 * Implementation.
 */
function createDefinition(identifier, url, labelOrOptions, title) {
    let options;
    if (labelOrOptions === undefined) {
        options = {};
    }
    else if (typeof labelOrOptions === 'string') {
        options = { label: labelOrOptions };
    }
    else {
        options = { ...labelOrOptions };
    }
    if (title !== undefined) {
        options.title = title;
    }
    return { type: 'definition', identifier, url, ...options };
}
exports.createDefinition = createDefinition;
/**
 * Type guard for Definition.
 *
 * @param node - The node to test.
 * @returns True if node is Definition.
 */
function isDefinition(node) {
    return node.type === 'definition';
}
exports.isDefinition = isDefinition;
/**
 * Type assertion for Definition.
 *
 * @param node - The node to test.
 */
function assertDefinition(node) {
    if (!isDefinition(node)) {
        throw new Error('Node is not a Definition');
    }
}
exports.assertDefinition = assertDefinition;
//# sourceMappingURL=Definition.js.map