"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
/*
 * Implementation.
 */
function createImage(url, altOrOptions, title) {
    let options;
    if (altOrOptions === undefined) {
        options = {};
    }
    else if (typeof altOrOptions === 'string') {
        options = { alt: altOrOptions };
    }
    else {
        options = { ...altOrOptions };
    }
    if (title !== undefined) {
        options.title = title;
    }
    return { type: 'image', url, ...options };
}
exports.createImage = createImage;
/**
 * Type guard for Image.
 *
 * @param node - The node to test.
 * @returns True if node is Image.
 */
function isImage(node) {
    return node.type === 'image';
}
exports.isImage = isImage;
/**
 * Type assertion for Image.
 *
 * @param node - The node to test.
 */
function assertImage(node) {
    if (!isImage(node)) {
        throw new Error('Node is not an Image');
    }
}
exports.assertImage = assertImage;
//# sourceMappingURL=Image.js.map