"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
/*
 * Implementation.
 */
function createImageReference(identifier, referenceType, altOrOptions, label) {
    let options;
    if (altOrOptions === undefined) {
        options = {};
    }
    else if (typeof altOrOptions === 'string') {
        options = { alt: altOrOptions };
    }
    else {
        options = { ...altOrOptions };
    }
    if (label !== undefined) {
        options.label = label;
    }
    return { type: 'imageReference', identifier, referenceType, ...options };
}
exports.createImageReference = createImageReference;
/**
 * Type guard for ImageReference.
 *
 * @param node - The node to test.
 * @returns True if node is ImageReference.
 */
function isImageReference(node) {
    return node.type === 'imageReference';
}
exports.isImageReference = isImageReference;
/**
 * Type assertion for ImageReference.
 *
 * @param node - The node to test.
 */
function assertImageReference(node) {
    if (!isImageReference(node)) {
        throw new Error('Node is not an ImageReference');
    }
}
exports.assertImageReference = assertImageReference;
//# sourceMappingURL=ImageReference.js.map