"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
/* eslint-enable @typescript-eslint/unified-signatures */
/*
 * Implementation.
 */
function createLink(url, titleOrChildrenOrOptions, children) {
    let options;
    if (titleOrChildrenOrOptions === undefined) {
        options = { children: [] };
    }
    else if (typeof titleOrChildrenOrOptions === 'string') {
        options = { title: titleOrChildrenOrOptions, children: [] };
    }
    else if (Array.isArray(titleOrChildrenOrOptions)) {
        options = { children: titleOrChildrenOrOptions };
    }
    else {
        options = { ...titleOrChildrenOrOptions, children: [] };
    }
    if (children) {
        options.children = children;
    }
    return { type: 'link', url, ...options };
}
exports.createLink = createLink;
/**
 * Type guard for Link.
 *
 * @param node - The node to test.
 * @returns True if node is Link.
 */
function isLink(node) {
    return node.type === 'link';
}
exports.isLink = isLink;
/**
 * Type assertion for Link.
 *
 * @param node - The node to test.
 */
function assertLink(node) {
    if (!isLink(node)) {
        throw new Error('Node is not a Link');
    }
}
exports.assertLink = assertLink;
//# sourceMappingURL=Link.js.map