"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
/* eslint-enable @typescript-eslint/unified-signatures */
/*
 * Implementation.
 */
function createListItem(checkedOrChildrenOrOptions, spreadOrChildren, children) {
    let options;
    if (checkedOrChildrenOrOptions === undefined) {
        options = { children: [] };
    }
    else if (typeof checkedOrChildrenOrOptions === 'boolean') {
        options = { checked: checkedOrChildrenOrOptions, children: [] };
    }
    else if (Array.isArray(checkedOrChildrenOrOptions)) {
        options = { children: checkedOrChildrenOrOptions };
    }
    else {
        options = { ...checkedOrChildrenOrOptions, children: [] };
    }
    if (spreadOrChildren !== undefined) {
        if (typeof spreadOrChildren === 'boolean') {
            options.spread = spreadOrChildren;
        }
        else {
            options.children = spreadOrChildren;
        }
    }
    if (children) {
        options.children = children;
    }
    return { type: 'listItem', ...options };
}
exports.createListItem = createListItem;
/**
 * Type guard for ListItem.
 *
 * @param node - The node to test.
 * @returns True if node is ListItem.
 */
function isListItem(node) {
    return node.type === 'listItem';
}
exports.isListItem = isListItem;
/**
 * Type assertion for ListItem.
 *
 * @param node - The node to test.
 */
function assertListItem(node) {
    if (!isListItem(node)) {
        throw new Error('Node is not a ListItem');
    }
}
exports.assertListItem = assertListItem;
//# sourceMappingURL=ListItem.js.map