"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const Break_1 = require("../Break");
const Delete_1 = require("../Delete");
const Emphasis_1 = require("../Emphasis");
const Footnote_1 = require("../Footnote");
const FootnoteReference_1 = require("../FootnoteReference");
const HTML_1 = require("../HTML");
const Image_1 = require("../Image");
const ImageReference_1 = require("../ImageReference");
const InlineCode_1 = require("../InlineCode");
const Strong_1 = require("../Strong");
const Text_1 = require("../Text");
/**
 * Type guard for StaticPhrasingContent.
 *
 * @param node - The node to test.
 * @returns True if node is StaticPhrasingContent.
 */
function isStaticPhrasingContent(node) {
    return (Text_1.isText(node) ||
        Emphasis_1.isEmphasis(node) ||
        Strong_1.isStrong(node) ||
        Delete_1.isDelete(node) ||
        HTML_1.isHTML(node) ||
        InlineCode_1.isInlineCode(node) ||
        Break_1.isBreak(node) ||
        Image_1.isImage(node) ||
        ImageReference_1.isImageReference(node) ||
        Footnote_1.isFootnote(node) ||
        FootnoteReference_1.isFootnoteReference(node));
}
exports.isStaticPhrasingContent = isStaticPhrasingContent;
/**
 * Type assertion for StaticPhrasingContent.
 *
 * @param node - The node to test.
 */
function assertStaticPhrasingContent(node) {
    if (!isStaticPhrasingContent(node)) {
        throw new Error('Node is not a StaticPhrasingContent');
    }
}
exports.assertStaticPhrasingContent = assertStaticPhrasingContent;
//# sourceMappingURL=StaticPhrasingContent.js.map